@@ -1,14 +1,14 @@
 """Definition of DependencyManager class."""
 
 import io
-import os
 import re
+from collections import OrderedDict
 from configparser import ConfigParser
 
 from .config import PackageManagers, ignored_packages_file
 from .core import get_package_managers_list, pkg_exists, process_alternatives
 from .exceptions import ConfigurationError, SetupFailedError
-from .installers import install_package
+from .installers import install_package_with_deps, uninstall_package
 
 
 class DependencyManager:
@@ -17,7 +17,7 @@ class DependencyManager:
     def __init__(
         self,
         config_file=None,
-        pkg_dict=None,
+        config_string=None,
         unique_id=None,
         interactive_initialization=True,
         use_gui=False,
@@ -46,15 +46,18 @@ class DependencyManager:
         self.extra_command_line = extra_command_line
         self.initialized = not interactive_initialization
         self.pkg_to_install = {}
-        self.pkg_to_install[PackageManagers.common] = {}
-        for pkg_mgr in get_package_managers_list():
-            self.pkg_to_install[PackageManagers[pkg_mgr]] = {}
+        self.pkg_to_uninstall = {}
+        for pkg_mgr in PackageManagers:
+            self.pkg_to_install[pkg_mgr] = {}
+            self.pkg_to_uninstall[pkg_mgr] = []
         self.optional_packages = []
         self.ignored_packages = []
+        self.priority_list = []
+        self.pkg_to_uninstall[PackageManagers.common] = []
         if config_file:
             self.load_file(config_file)
-        elif pkg_dict:
-            self.load_dict(pkg_dict)
+        elif config_string:
+            self.load_string(config_string)
 
     def validate_config(self):
         """
@@ -67,20 +70,42 @@ class DependencyManager:
 
     def load_file(self, config_file):
         """
-        Load the configuration file.
+        Load the configuration from a file.
 
         :param config_file: can be a string, a file-like object, or a path-like object
         :return: Nothing
         """
+        self.load_config(config_file, False)
+
+    def load_string(self, config_string):
+        """
+        Load the configuration from a string.
+
+        :param config_string: string containing the configuration
+        :return: Nothing
+        """
+        self.load_config(config_string, True)
+
+    def load_config(self, config, is_configuration_string=False):
+        """
+        Load the configuration.
+
+        :param config: can be a string, a file-like object, or a path-like object
+        :param is_configuration_string: True if the config is a string containing the configuration itself
+        :return: Nothing
+        """
         parser = ConfigParser(comment_prefixes=('#',))
 
         # preserve capitalization of options
         parser.optionxform = lambda option: option
 
-        if isinstance(config_file, io.IOBase):
-            parser.read_file(config_file)
+        if is_configuration_string:
+            parser.read_string(config)
         else:
-            parser.read(config_file)
+            if isinstance(config, io.IOBase):
+                parser.read_file(config)
+            else:
+                parser.read(config)
 
         # load global configuration
         if parser.has_section('Global'):
@@ -112,6 +137,25 @@ class DependencyManager:
                 # split the list at commas and newlines
                 self.optional_packages = [x.strip() for x in re.split('[\n,]', opt_packages)]
 
+            if parser.has_option('Global', 'priority'):
+                priority_str = parser.get('Global', 'priority').strip()
+                # split the list at commas and newlines
+                self.priority_list = [x.strip() for x in re.split('[\n,]', priority_str)]
+
+            package_manager_suffixes = [''] + [
+                f'.{package_manager.lower()}' for package_manager in get_package_managers_list()
+            ]
+
+            for package_manager_suffix in package_manager_suffixes:
+                if package_manager_suffix == '':
+                    dict_key = PackageManagers.common
+                else:
+                    dict_key = PackageManagers[package_manager_suffix[1:]]
+                if parser.has_option('Global', 'uninstall' + package_manager_suffix):
+                    uninstall_str = parser.get('Global', 'uninstall' + package_manager_suffix).strip()
+                    # split the list at commas and newlines
+                    self.pkg_to_uninstall[dict_key] = [x.strip() for x in re.split('[\n,]', uninstall_str)]
+
         if parser.has_section('Packages'):
             self.pkg_to_install[PackageManagers.common] = {}
             for package, alternatives in parser.items('Packages'):
@@ -125,25 +169,12 @@ class DependencyManager:
             # sections are always capitalized
             section_name = package_manager_name.capitalize()
             package_manager = PackageManagers[package_manager_name]
+            self.pkg_to_install[package_manager] = {}
             if parser.has_section(section_name):
-                self.pkg_to_install[package_manager] = {}
                 for package, alternatives in parser.items(section_name):
-                    self.pkg_to_install[package_manager][package] = process_alternatives(alternatives.split('\n'))
-
-        self.validate_config()
-
-    def load_dict(self, pkg_dict):
-        """
-        Load the configuration from a dictionary.
-
-        :param pkg_dict: dictionary in the format {module_name: [list, of, alternatives, with, platform, markers]}
-        :return: Nothing
-        """
-        self.pkg_to_install[PackageManagers.common] = {}
-
-        for package, alternatives in pkg_dict.items():
-            alternatives = process_alternatives(alternatives)
-            self.pkg_to_install[PackageManagers.common][package] = alternatives
+                    self.pkg_to_install[package_manager][package] = process_alternatives(
+                        re.split('[\n,]', alternatives)
+                    )
 
         self.validate_config()
 
@@ -159,6 +190,13 @@ class DependencyManager:
         except FileNotFoundError:
             self.ignored_packages = []
 
+        # remove packages that are not optional anymore
+        for package in self.ignored_packages[:]:
+            if package not in self.optional_packages:
+                self.ignored_packages.remove(package)
+
+        self.save_ignored_packages()
+
     def clear_ignored_packages(self):
         """
         Clear the ignore list.
@@ -166,7 +204,7 @@ class DependencyManager:
         :return: Nothing
         """
         self.ignored_packages = []
-        with os.open(ignored_packages_file(self.unique_id), 'w', mode=0o644) as fd:
+        with open(ignored_packages_file(self.unique_id), 'w', encoding='utf-8') as fd:
             fd.write('')
 
     def mark_ignored(self, package):
@@ -176,11 +214,33 @@ class DependencyManager:
         :param package: package to ignore
         :return: Nothing
         """
-        print(f'Ignoring {package}')
         self.ignored_packages.append(package)
-        with os.open(ignored_packages_file(self.unique_id), 'w', mode=0o644) as fd:
+        self.save_ignored_packages()
+
+    def save_ignored_packages(self):
+        """
+        Save the list of ignored packages.
+
+        :return: Nothing
+        """
+        with open(ignored_packages_file(self.unique_id), 'w', encoding='utf-8') as fd:
             fd.write('\n'.join(self.ignored_packages))
 
+    def sort_packages(self, pkg_dict):
+        """
+        Sort the packages according to the priority list.
+
+        :param pkg_dict: dictionary of packages to sort
+        :return: sorted list of packages
+        """
+        if not self.priority_list:
+            return
+
+        # the first package in the priority list will end up as the first package in the ordered dict
+        for package in self.priority_list[::-1]:
+            if package in pkg_dict:
+                pkg_dict.move_to_end(package, last=False)  # move package to the beginning
+
     def install_interactive(self, force_optional=False):
         """
         Install the packages.
@@ -192,8 +252,19 @@ class DependencyManager:
         if not self.initialized:
             self.show_initialization()
 
+        # uninstall packages
+        pkg_to_uninstall_list = (
+            self.pkg_to_uninstall[PackageManagers.common] + self.pkg_to_uninstall[self.package_manager]
+        )
+        for pkg in pkg_to_uninstall_list:
+            self.uninstall_package(pkg, interactive=True)
+
         # compatible with python 3.6
-        pkg_to_install = {**self.pkg_to_install[PackageManagers.common], **self.pkg_to_install[self.package_manager]}
+        pkg_to_install = OrderedDict(
+            {**self.pkg_to_install[PackageManagers.common], **self.pkg_to_install[self.package_manager]}
+        )
+
+        self.sort_packages(pkg_to_install)
 
         if force_optional:
             self.clear_ignored_packages()
@@ -215,22 +286,56 @@ class DependencyManager:
         :param install_optional: if True, optional packages will be installed
         :return: Nothing
         """
+        # uninstall packages
+        pkg_to_uninstall_list = (
+            self.pkg_to_uninstall[PackageManagers.common] + self.pkg_to_uninstall[self.package_manager]
+        )
+        for pkg in pkg_to_uninstall_list:
+            self.uninstall_package(pkg, interactive=False)
+
         # compatible with python 3.6
-        pkg_to_install = {**self.pkg_to_install[PackageManagers.common], **self.pkg_to_install[self.package_manager]}
+        pkg_to_install = OrderedDict(
+            {**self.pkg_to_install[PackageManagers.common], **self.pkg_to_install[self.package_manager]}
+        )
+
+        self.sort_packages(pkg_to_install)
 
         for package, alternatives in pkg_to_install.items():
             if not pkg_exists(package):
                 if install_optional or package not in self.optional_packages:
-                    while not install_package(
-                        self.package_manager, alternatives[0], self.install_local, self.extra_command_line
+                    while not install_package_with_deps(
+                        self.package_manager,
+                        next(iter(alternatives.keys())),
+                        next(iter(alternatives.values())),
+                        self.install_local,
+                        self.extra_command_line,
                     ):
                         print(f'Error installing {package}. Trying a different alternative')
-                        alternatives.pop(0)
-                        if not alternatives and package not in self.optional_packages:
+                        alternatives.popitem(0)
+                        if not alternatives:
+                            if package in self.optional_packages:
+                                print(f'No more alternatives for {package}. Not failing because it is optional')
+                                break
                             raise SetupFailedError(f'Failed to install {package}')
-                        else:
-                            print(f'No more alternatives for {package}. Not failing because it is optional')
-                            break
+
+    def uninstall_package(self, package, interactive=True):
+        """
+        Uninstall a package.
+
+        :param package: package to uninstall
+        :param interactive: if True, the user will be asked to confirm the uninstallation
+        :return: Nothing
+        """
+        if interactive:
+            if self.use_gui:
+                from .gui import notify_uninstall
+            else:
+                from .cli import notify_uninstall
+
+            if not notify_uninstall(package):
+                raise SetupFailedError(f'Uninstallation of {package} aborted by user')
+
+        uninstall_package(self.package_manager, package)
 
     def install_package(self, package, alternatives, optional=False):
         """
@@ -247,13 +352,17 @@ class DependencyManager:
         if not self.initialized:
             self.show_initialization()
 
-        source = self.select_alternative(package, alternatives, optional)
+        alternative_names = list(alternatives.keys())
+        source = self.select_alternative(package, alternative_names, optional)
         if optional and source is None:
             self.mark_ignored(package)
             return True
-        alternatives.remove(source)
+        dependencies = alternatives[source]
+        del alternatives[source]
 
-        return install_package(self.package_manager, source, self.install_local, self.extra_command_line)
+        return install_package_with_deps(
+            self.package_manager, source, dependencies, self.install_local, self.extra_command_line
+        )
 
     def show_initialization(self):
         """
